/*
Achtung!!! Damit das Display niemals ber 4 Volt abbekommt, 
drfen die I2C-Ausgnge NIEMALS hat gegen + schalten, 
sondern mssen immer nur gegen GND schalten und fr high auf hochohmig geschaltet werden.

Mu mit RC5.c compiliert werden.

Am Tiny44 sind drei LEDs angeschlossen:

zwei eckige an 
OC0A = PB2 
und 
OC0B = PA7

eine kleine runde an PA1

Pin1					Pin7
VCC	PB0	PB1	PB3	PB2	PA7	PA6

GND	PA0	PA1	PA2	PA3	PA4	PA5
Pin14					Pin8


___________________________________________
zustzlich angeschlossen am Tiny44:
ADC an Pins 11 und 12 sind Potis angeschlossen PA1, PA2

Schaltung umgebaut: 
RC5 an PB1, PWM-LEDs an PB2 und PA7, Potis an PA0 und PA1, Kontroll-LED an PB0. SDA an PA2 und SCL an PA3. 

Version11: gekrzt, um das ACK auf dem Oszi sehen zu knnen. sieht man tatschlich.
Version12: es soll in GETACK eine andere Reihenfolge ausprobiert werden. Das Einlesen soll erfolgen, wenn SCL sich von L -> H ndert.
*/

#include <avr/io.h>
//#include <stdlib.h>		// wird nur fr die Funktion "iota" bei der LCD-Ausgabe gebraucht.
#include <util/delay.h>
#include "RC5.h"


#define SCLFreq 10		//hiermit getestet, geht, noch rasender schneller!
#define PORTout PORTA
#define PORTIn PINA
#define PORTDirection DDRA

#define SCL PA3
#define SDA PA2

//Definitionen fr die I2C-Ausgangs-Pins an 3,3Volt:
//high bedeutet, Pin ist ein Eingang und lt den Pullup-Widerstand hochziehen nach 3,3 Volt
//low bedeutet, der PIN sei ein Ausgang und der Ausgang sei auf low.
//Im Prinzip knnte man beide I2C-Pins immer auf low lassen und fr low auf Ausgang schalten und fr high auf Eingang schalten.
#define SCL_low	 	PORTout &= ~ (1<<SCL);	PORTDirection |= (1<<SCL);	//Datenleitung auf low und auf Ausgang umschalten
#define SCL_high	PORTDirection &= ~(1<<SCL);							//auf Eingang schalten -->> Pullup zieht auf high = 3,3V
#define SDA_low 	PORTout &= ~ (1<<SDA);	PORTDirection |= (1<<SDA);
#define SDA_high	PORTDirection &= ~(1<<SDA);


#define read  			0x01
#define write  			0

//zur Ausgabe der zweizeiligen Fonts 12x16 und 10x16
#define	oben	1
#define unten	0

/* gehrt zum Display */
#define Adresse_control		0x38		//Control-Adresse	I2C-Display
#define Adresse_data		0x39		//Daten				I2C-Display
#define kontrast 			0xC0		//Kontrast Wert ausprobieren, 0xC0 ist default


/* Display-Kommandos A0=low*/
#define col_low        0
#define col_hi      0x10
#define temp_comp   0x24
#define pwr_ctrl    0x28
#define adv_pctl    0x30   // danach Wert ?
#define scrolline   0x40
#define set_page    0xB0
#define poti        0x81   // danach Wert, default 0xC0 zur Kontrasteinstellung
#define partial     0x84
#define ramadctl    0x88
#define framerate   0xA0
#define allpixel    0xA5	//mit 0xA4 sind Pixel aus
#define invers      0xA6
#define disp_ena    0xAF	//mit 0xAE ist Display aus
#define map_ctrl    0xC0
#define reset       0xE2
#define nop         0xE3
#define bias        0xE8	// Bias-ratio = 6 ist der default-Wert. Damit lt sich das Display "ganz dunkel schwarz" steuern
#define com_end     0xF1    // danach Wert 0..127, default 63	//COM_end einstellen: 0x15 = 21. Das Display hat also 21 Pixels Hhe
#define part_start  0xF2    // danach Wert 0..127, default 0
#define part_end    0xF3    // danach Wert 0..127, default 63

/* gehrt zum Display */
const uint8_t UC1601_init[13] = {		//die sieben Bytes zum Initialisieren des Displays von hier auslesen und ans Display bertragen.
0x24,//|0b00,		//Set Temperature Comp. TC[1:0]
// falsch 0x28,//|0b110,		//Set Power Control PC[2:0]		bedeutet internal VLCD x 7
0x2C,//|0b110,		//Set Power Control PC[2:0]		bedeutet internal VLCD x 7
0x40,//|0b000000,	//Set Scroll Line SL[5:0]	
//0x81, gnstiger Wert	//Set VBIAS Potentiometer PM [7:0] (Double byte command)
0x81,				//bei 0x50 begint das Display grau zu werden. Bei 0x80 sind Punkte schwarz. Set VBIAS Potentiometer PM [7:0] (Double byte command)
0xC0,				//0xC0 gnsiger Kontrastwert
0x84,//|0b0,		//Set Partial Display Enable LC [4]
0x88,//|0b001,		//Set RAM Address Control AC [2:0]
0xA0,//|0b0 ,		//Set Frame Rate LC [3]
0xC0,//|0b100,		//Set LCD Control LC[2:1]
0xE8,//|0b00, 		//Set Bias Ratio BR [1:0]	Bias-ratio = 6 ist der default-Wert. Damit lt sich das Display "ganz dunkel schwarz" steuern
0xF1,				//Set CEN [6:0] (Double byte command)
0x15,				///auf diesen Wert COM_end einstellen: 0x15 = 21. Das Display hat also 21 Pixels Hhe
0xAF//|0b1			//Set Display Enable DC[2]		0xAe = off, 0xAF = on
};


//uint8_t zaehlvariable = 0;
uint16_t zaehlvariable = 0;
//char s[10];		//Array fr ITOA
unsigned int n;							//RC5-Empfang
char s[10];								//Array fr ITOA
uint8_t	ackempfangen = 5;


void WaitHalfCycle(void);
char GetAck(void);
void Display_enable (void);
unsigned char ReadByte_Display(void);
void Start_Display(void);
void Stop_Display(void);
/*
void LCD_all_on (void);
void LCD_all_off (void)	;
void LCD_Softreset (void);
void Display_display_clear (void);
void Display_Scroll (uint8_t y);
void Display_GoTo (uint8_t x, uint8_t y)	;
void Display_WriteInt(int16_t zahl);
void Display_WriteHex(unsigned char zahl);
void Display_WriteString8x8(char * s);
void Display_WriteChar8x8(char x);
void Display_WriteChar6x8(char x);
*/


void WaitHalfCycle(void)
{

//vier NOP-Befehle bentigen 0,5 s bei 8 MHz und 1s bei 4 MHz
asm volatile ("nop");
asm volatile ("nop");
asm volatile ("nop");
asm volatile ("nop");	
asm volatile ("nop");
asm volatile ("nop");
asm volatile ("nop");
asm volatile ("nop");
asm volatile ("nop");
asm volatile ("nop");
}

//Achtung. dieses GetACK wurde gendert gem "24C08_15.c" Es ist jetzt mglich, SDA auf Eingang umzuschalten, ohne da das Display streikt.
//Allerdings ist ACK immer 0, das nicht sein kann. Die 1 wird nicht erkannt, weil die Spannung zu niedrig ist. Falsch.
//Siehe Seite 200 Pin theshold: braucht bei 5V ber 2,6V, um 1 zu lesen.
//selbst mit beliebig langer Wartezeit vor der Abfrage des PINs erscheint keine 1.
/*
char GetAck(void)	//falls Ack erscheint, ist das ein "low". Falls NoAck -->> high
{
	uint8_t status=1;	//1 bedeutet, NOACK

	SCL_low;						//PORTout &= ~(1<<SCL);				//SCL auf low	Ab hier mu der Slave das ACK-Bit-bereit stellen!! GND-Linie ist etwas erhht, falls Slave mit ACK antwortet.
	PORTDirection &= ~(1<<SDA);	//Datenleitung auf Eingang umschalten		 Display reagiert nicht mehr
	WaitHalfCycle();

	SCL_high;						//PORTout |= (1<<SCL);				//SCL auf high
	PORTDirection &= ~(1<<SDA);
	WaitHalfCycle();

	if((PORTIn & (1<<SDA)) == 0)	//originalzeile
//	if((PINA & 0b00000100) == 0)	
		{							//Port einlesen und vergleichen, ob die eine Stelle 1 ist, die den SDA-Pin darstellt
		status=0;
		}

	SCL_low;						//PORTout &=~ (1<<SCL);	mu dazu	//SCL auf low

	return (status);
}
*/


//Achtung!!! hier mal eine neue Version des GETACK ausprobieren, mit genderter Reichenfolge der Befehle.
//Das Einlesen an sDA soll erfolgen bei der STEIGENDEN Flanke von SCL!!!
//Bisher wurde SCL von H -> L geschaltet und dann erst eingelesen. Dann ist SDA schon wieder auf 1 und es wird immer 1 gelesen.
char GetAck(void)	//falls Ack erscheint, ist das ein "low". Falls NoAck -->> high
{
	uint8_t status;	//1 bedeutet, NOACK
	status = 1;

	SCL_low;						//PORTout &= ~(1<<SCL);				//SCL auf low	Ab hier mu der Slave das ACK-Bit-bereit stellen!! GND-Linie ist etwas erhht, falls Slave mit ACK antwortet.
	PORTDirection &= ~(1<<SDA);	//Datenleitung auf Eingang umschalten		 Display reagiert nicht mehr
//	PORTout |= (1<<SDA);			//unbedeutend   Ausgangsport der Datenleitung auf low schalten, damit nicht immer 1 gelesen wird oder damit der PulluP ausgeschaltet ist???
	WaitHalfCycle();

//	if((PORTIn & (1<<SDA)) == 0)	//Lesen, so lange SCL 0 Low ist. also vor der L -> H Flanke.
//	if((PINA & (1<<PA2)) == 0)	//Lesen, so lange SCL 0 Low ist. also vor der L -> H Flanke.
	if((PINA & (1<<PA2)) == 0)	
	{							//Port einlesen und vergleichen, ob die eine Stelle 1 ist, die den SDA-Pin darstellt
	status=0;
	}
	
	SCL_high;						//PORTout |= (1<<SCL);				//SCL auf high
	WaitHalfCycle();

	SCL_low;						//PORTout &=~ (1<<SCL);	mu dazu	//SCL auf low

	return (status);
}


void Start_Display(void)
{
	SDA_high;				//PORTDirection &= ~(1<<SDA);		//Datenleitung auf Eingang umschalten -->> bedeutet high 3,3V
	SCL_high;
	WaitHalfCycle();
	SDA_low;				//PORTout &= ~ (1<<SDA);
	WaitHalfCycle();
	SCL_low;//hier noch clock auf low setzen
}
//A high to low transition on the SDA line while SCL is high is defined to be a START condition or a repeated start condition.
//A STOP condition is defined by a low to high transition on the SDA line while the SCL is high.


void Stop_Display(void)		//OK, funktioniert!
{
	int x;						//volatile int x;
	SCL_high;					//PORTout |= (1<<SCL);//high
	SDA_low;					//PORTout &= ~(1<<SDA);	//low

	WaitHalfCycle();
	SDA_high;					//PORTDirection &= ~(1<<SDA);	//Datenleitung auf Eingang umschalten -->> bedeutet high 3,3V
	for(x=0;x<=250;x++){WaitHalfCycle();}
}

unsigned char Display_send_add_rw(unsigned char addr_byte, uint8_t rw)
{
	uint8_t x;

	//*******************************
	//shift address one bit left
	addr_byte = addr_byte << 1;				//diesen Befehl weg lassen oder dazu tun, Adresse mu um 1 nach links verschoben werden laut Bild Seite 27/31
	//*******************************
	
	//set RW-Bit, if necessary
	addr_byte |= rw;

	for(x=0;x<=7;x++)
		{
		SCL_low;							//PORTout &=~ (1<<SCL);
		WaitHalfCycle();
		switch(((addr_byte << x)& 0x80))
			{
			case 0x00:
			SDA_low;						//PORTout &= ~(1<<SDA);
			break;
			case 0x80:
			SDA_high;						//PORTout |= (1<<SDA);
			break;
			}
		SCL_high;							//PORTout |= (1<<SCL);		//vielleicht hier davor noch ein NOP einfgen
		WaitHalfCycle();
		}
	return(GetAck());
}


unsigned char ReadByte_Display(void)		//wichtig!!! zwei Varianten schreiben: eine lt Master ACK senden, andere lt Master NOAck senden
{
	int x;
	unsigned char data=0x00;

	for(x=0;x<=7;x++)
	{
		SCL_low;						//PORTout &=~ (1<<SCL);
		WaitHalfCycle();
		SCL_high;						//PORTout |= (1<<SCL);
		WaitHalfCycle();

		if((PORTIn & (1<<SDA)) == 0x00)
			{
			data &= ~0x01;
			}
		else data |= 0x01;
		if(x<7)
			{
			data=(data << 1) ;
			}
	}
	GetAck();					
	return(data);
}


unsigned char Display_send_byte(unsigned char data)
{
	int x;

	for(x=0;x<=7;x++)
	{
	SCL_low;						//PORTout &=~ (1<<SCL);	//SCL auf low
	WaitHalfCycle();
	switch(((data << x)& 0x80))
		{
		case 0x00:
		SDA_low;					//PORTout &= ~(1<<SDA);	//SDA auf low
		break;
		case 0x80:
		SDA_high;					//PORTout |= (1<<SDA);	//SDA auf high
		break;
		}
	SCL_high;						//PORTout |= (1<<SCL);	//SCL auf high		//vielleicht hier davor noch ein NOP einfgen
	WaitHalfCycle();
	}
	return(GetAck());
}


void Display_enable (void)				//wird aufgerufen von "Display_init"
{										//es fehlt noch die Einstellung "Schwarze Zahlen auf hellem Hintergrund"

	Start_Display();
	Display_send_add_rw (Adresse_control,write);	//zuerst die Adresse und Schreiben senden.  
/************************************/	
	uint8_t i;
	for(i=0;i<13;i++) 
	{

	Display_send_byte(UC1601_init[i]);		//nacheinander die Bytes aus dem Array ins Display schreiben als Kommandos
	
	}
/************************************/	
	Stop_Display();
//	_delay_ms(10);	
	
}


int main(void)
{


	CLKPR = 0x80;			//CLKPCE auf 1 setzen und alle anderen Bits auf 0, um die Funktionalitt zu aktivieren
//	CLKPR = 1 << CLKPS2;	//8 MHz / 16 = 500 kHz 	
//	CLKPR = 0;				//8 MHZ direkt 				<<--<<--  dieses verwenden ohne RC5
	CLKPR = 1 << CLKPS0;	//8 MHZ / 2 = 4 MHz 		<<<---<<<<-- diese verwenden fr 4 MHz
//	CLKPR = 1 << CLKPS1;	//8 MHZ / 4 = 1 MHz 

	DDRA = 0x80;	//PA7 als Ausgang schalten fr LEDs
	DDRB = 0x05;	//PB2 und PB0 als Ausgang schalten fr LEDs
	PORTB = 0x02;	//PB1 Pullup einschalten fr RC5-Empfnger
//	PORTout &= ~((1<<SCL)|(1<<SDA));	//soll beide I2C-Pins auf low schalten, werden erst im Unterprogramm auf Ausgang gestellt


//hier Einstellungen fr RC5-Empfang. Timer-IRQ darf man immer frei schalten. Timer-Prescaler passt nur fr RC5. Mu man wieder deaktivieren.
	TCCR0B = 1<<CS02;			//internal clock divide by 256 fr RC5
//	TCCR0B = 1<<CS00;			//internal clock divide by 1
//	TCCR0B = (1<<CS02)|(1<<CS00);	//internal clock divide by 1024 -->> LEDs leuchten abwechselnd, passt fr Push-Pull-Betrieb
	TIMSK0 = 1<<TOIE0;			//enable timer interrupt
	sei();
//Ende RC5-Einstellungen	Falls RC5 funktioniert, strt diese Zeile in RC5.c:  TCNT0 = -2;

    while (1) 
	{

	PORTB ^= 1<<PB0;	// LED toggeln

	zaehlvariable++;

//***********************************************


	Start_Display();
//	ackempfangen = Display_send_add_rw (0x40 ,write);	//Adresse geben und Schreiben bestimmen
	ackempfangen = Display_send_add_rw (Adresse_control ,write);	//Adresse geben und Schreiben bestimmen		0111 0000 ACK = 0x38 + Write
	Display_send_byte(scrolline | (2 & 0x3F));	//Scroll 0 bedeutet keine Verscheibung -->>> idealer Befehl, um das GETACK auszuprobieren  0100 0010 ACK = Scrolline + 2
	Stop_Display();	


	PORTB ^= 1<<PB0;	// LED toggeln
	

	if  ( zaehlvariable >= 1500 ) 
		{ 
		//Display_GoTo (0, 2);	// x = 0...131, y = 0...63	
		//ItoA(ackempfangen, s, 1);		//Ausgabe 0 oder 1, je nachdem, ob der Slave mit ACK antwortet oder nicht. Es erscheint immer 0, auch dann, wenn auf Oszi gar kein Ack erscheint, das der Slave ausgibt. 
		//Display_WriteString8x8(s);		//Das Lesen des ACK-Bits ist also immer noch falsch.
		zaehlvariable = 0;
		}
	ackempfangen = 27;

	
    }

    return 0;
}
